// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:architecture_example/domain/entities/photo.dart';
import 'package:architecture_example/domain/repositories/photo_repository.dart';
import 'package:architecture_example/domain/usecases/photos/params/query_parameters.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';

class OpenAlbum extends AsyncUseCase<QueryParameters, List<Photo>> {

  OpenAlbum(this._photoRepository);
  final PhotoRepository _photoRepository;

  @override
  FutureOrResult<List<Photo>> execute(QueryParameters? params) {
    final photos = _photoRepository.getPhotosFromAlbum(
      params!.albumId,
      start: params.start,
      limit: params.limit,
    );

    return photos;
  }

  @override
  FutureOr<void> onStart(QueryParameters? params) {
    if (params == null) {
      throw ClientException('params cannot be null');
    }
  }
}
