// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:architecture_example/core/dependency_injection/get_it.dart';
import 'package:architecture_example/data/repositories/photo_repository_impl.dart';
import 'package:architecture_example/domain/data_sources/local/favorite_local_data_source.dart';
import 'package:architecture_example/domain/data_sources/remote/album_remote_data_source.dart';
import 'package:architecture_example/domain/data_sources/remote/photo_remote_data_source.dart';
import 'package:architecture_example/domain/repositories/photo_repository.dart';
import 'package:architecture_example/presentation/features/albums/albums.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class App extends StatelessWidget {
  const App({super.key});

  @override
  Widget build(BuildContext context) => MultiRepositoryProvider(
        providers: [
          RepositoryProvider<PhotoRepository>(
            create: (_) => PhotoRepositoryImpl(
              getIt<PhotoRemoteDataSource>(),
              getIt<AlbumRemoteDataSource>(),
              getIt<FavoriteLocalDataSource>(),
            ),
          ),
        ],
        child: const MaterialApp(
          title: 'Demo',
          debugShowCheckedModeBanner: false,
          home: Albums(),
        ),
      );
}
