// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:example_router/core/constants/form_field.dart';
import 'package:example_router/core/utils/custom_password.dart';
import 'package:example_router/presentation/features/edit_profile/blocs/edit_profile_cubit.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class _EmailInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilderTextController<EditProfileCubit, String?, void>(
      field: AuthFormField.email,
      builder: ((context, cubit, state, field, input, controller, extra) {
        return TextField(
          onChanged: (email) => cubit.dataChanged<String?>(
              AuthFormField.email, Email.dirty(email)),
          keyboardType: TextInputType.emailAddress,
          controller: controller,
          decoration: InputDecoration(
            labelText: 'Email',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid email' : null,
          ),
        );
      }),
    );
  }
}

class _OldPasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<EditProfileCubit>(
      field: AppFormField.oldPassword,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (pwd) => cubit.dataChanged<String?>(
              AppFormField.oldPassword, CustomPassword.dirty(pwd)),
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'Old Password',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid password' : null,
          ),
        );
      }),
    );
  }
}

class _NewPasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<EditProfileCubit>(
      field: AuthFormField.password,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (pwd) => cubit.dataChanged<String?>(
              AuthFormField.password, CustomPassword.dirty(pwd)),
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'New Password',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid password' : null,
          ),
        );
      }),
    );
  }
}

class _EditButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<EditProfileCubit>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated ? () => cubit.submit() : null,
                child: const Text('Edit profile'),
              );
      }),
    );
  }
}

class EditProfileForm extends StatelessWidget {
  const EditProfileForm({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocListener<EditProfileCubit, FormDataState>(
      listener: (context, state) {
        if (state.status == FormStatus.submissionFailure) {
          ScaffoldMessenger.of(context)
            ..hideCurrentSnackBar()
            ..showSnackBar(
              SnackBar(content: Text(state.errorMessage ?? 'Edit Failure')),
            );
        }
      },
      child: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            _EmailInput(),
            const SizedBox(height: 8),
            _OldPasswordInput(),
            const SizedBox(height: 8),
            _NewPasswordInput(),
            const SizedBox(height: 16),
            _EditButton(),
          ],
        ),
      ),
    );
  }
}
