// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class AuthenticationCacheDataSourceImpl<T extends Object>
    extends AuthenticationCacheDataSource<T> {
  AuthenticationCacheDataSourceImpl();
  Account? _account;
  T? _data;

  @override
  Future<void> storeAccount(Account? account) async {
    _account = account;
  }

  @override
  Future<void> storeData(T? data) async {
    _data = data;
  }

  @override
  Future<Account> loadAccount() async {
    if (_account.isNotNull) {
      return _account!;
    }
    throw ClientException('Cached account is invalid');
  }

  @override
  Future<T> loadData() async {
    if (_data.isNotNull) {
      return _data!;
    }
    throw ClientException('Cached data is invalid');
  }

  @override
  Future<void> destroy() async {
    _data = null;
    _account = null;
  }

  @override
  Future<AccountWrapper<T>> load() async {
    if (_account.isNull) {
      throw ClientException('Cached account is invalid');
    }
    return AccountWrapperModel(_account, _data);
  }
}
