// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/src/features/email_verification/cubit/email_verification_cubit.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class EmailVerificationBuilder<Extra> extends StatelessWidget {
  const EmailVerificationBuilder({
    required this.verified,
    required this.notVerified,
    required this.onError,
    this.customBuilder,
    super.key,
  });

  final Widget Function(BuildContext context) verified;
  final Widget Function(BuildContext context) notVerified;
  final Widget Function(
    BuildContext context,
    FormStatus status,
    String? errorMessage,
  ) onError;

  final Widget Function(BuildContext context, EmailVerificationState)?
      customBuilder;

  @override
  Widget build(BuildContext context) =>
      BlocBuilder<EmailVerificationCubit<Extra>, EmailVerificationState>(
        builder: (context, state) {
          if (customBuilder != null) {
            return customBuilder!(context, state);
          }

          if (state.status.isSubmissionFailure ||
              state.status.isSubmissionCanceled) {
            return onError(
              context,
              state.status,
              state.errorMessage,
            );
          }

          if (state.isVerified) {
            return verified(context);
          }
          return notVerified(context);
        },
      );
}
