// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter_test/flutter_test.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

void main() {
  final WyattForm form = WyattFormImpl(
    [
      FormInput(AuthFormField.email, const Email.pure()),
      FormInput(AuthFormField.password, const Password.pure())
    ],
    name: AuthFormName.signInForm,
  );

  group('SignInState', () {
    test('supports value comparisons', () {
      expect(
        SignInState(
          form: form,
        ),
        SignInState(form: form),
      );
    });

    test('returns same object when no properties are passed', () {
      expect(
        SignInState(form: form).copyWith(),
        SignInState(form: form),
      );
    });

    test('returns object with updated status when status is passed', () {
      expect(
        SignInState(form: form).copyWith(status: FormStatus.invalid),
        SignInState(
          form: form,
          status: FormStatus.invalid,
        ),
      );
    });
  });
}
