// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:bloc_helper_example/counter/cubit/counter_cubit.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';

/// Important, this page assumes that the [CounterCubit] is provided.
class CounterConsumerPage
    extends CubitConsumerScreen<CounterCubit, CounterState> {
  const CounterConsumerPage({super.key});

  @override
  Widget onBuild(BuildContext context, CounterState state) {
    final textTheme = Theme.of(context).textTheme;
    return Scaffold(
      appBar: AppBar(title: const Text('Counter with Consumer')),
      body: Center(child: Text('${state.count}', style: textTheme.headline2)),
      floatingActionButton: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: <Widget>[
          FloatingActionButton(
            heroTag: null,
            key:
                const Key('counterConsumerView_increment_floatingActionButton'),
            child: const Icon(Icons.add),
            onPressed: () => bloc(context).increment(),
          ),
          const SizedBox(height: 8),
          FloatingActionButton(
            heroTag: null,
            key:
                const Key('counterConsumerView_decrement_floatingActionButton'),
            child: const Icon(Icons.remove),
            onPressed: () => bloc(context).decrement(),
          ),
        ],
      ),
    );
  }
}
