// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:bloc_helper_example/counter/bloc/counter_bloc.dart';
import 'package:bloc_helper_example/counter/repository/counter_repository.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';

class CounterProviderPage
    extends BlocProviderScreen<CounterBloc, CounterEvent, CounterState> {
  const CounterProviderPage({super.key});

  @override
  CounterBloc create(BuildContext context) =>
      CounterBloc(repo<CounterRepository>(context));

  @override
  Widget builder(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;
    return Scaffold(
      appBar: AppBar(title: const Text('Counter with Provider')),
      body: Center(child: BlocBuilder<CounterBloc, CounterState>(
        builder: (context, state) {
          return Text('${state.count}', style: textTheme.headline2);
        },
      )),
      floatingActionButton: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: <Widget>[
          FloatingActionButton(
            heroTag: null,
            key:
                const Key('counterProviderView_increment_floatingActionButton'),
            child: const Icon(Icons.add),
            onPressed: () => add(context, CounterIncrement()),
          ),
          const SizedBox(height: 8),
          FloatingActionButton(
            heroTag: null,
            key:
                const Key('counterProviderView_decrement_floatingActionButton'),
            child: const Icon(Icons.remove),
            onPressed: () => add(context, CounterDecrement()),
          ),
        ],
      ),
    );
  }
}
