// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';

abstract class CrudDataSource<Model> extends BaseDataSource {
  Future<void> create(Model object, {String? id});

  Future<Model?> get(String id);

  Future<List<Model?>> getAll();

  Future<void> update(
    String id, {
    Model? object,
    Map<String, dynamic>? raw,
  });

  Future<void> updateAll(Map<String, Object?>? data);

  Future<void> delete(String id);

  Future<void> deleteAll();

  Future<List<Model?>> query(List<QueryInterface> conditions);

  Stream<List<Model?>> stream({
    String? id,
    List<QueryInterface>? conditions,
    bool includeMetadataChanges = false,
  });
}
