// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/object_model.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';
import 'package:wyatt_crud_bloc/src/domain/repositories/crud_repository.dart';

class Query<Model extends ObjectModel>
    extends AsyncUseCase<List<QueryInterface>, List<Model?>> {

  Query(this._crudRepository);
  final CrudRepository<Model> _crudRepository;

  @override
  FutureOr<void> onStart(List<QueryInterface>? params) {
    if (params == null) {
      throw ClientException('List of conditions cannot be null.');
    }
  }

  @override
  FutureOrResult<List<Model?>> execute(List<QueryInterface>? params) =>
      _crudRepository.query(params!);
}
