// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_form_bloc/src/core/enums/validation_error.dart';
import 'package:wyatt_form_bloc/src/data/form/wyatt_form_impl.dart';
import 'package:wyatt_form_bloc/src/domain/entities/form_input.dart';
import 'package:wyatt_form_bloc/src/domain/form/wyatt_form.dart';
import 'package:wyatt_form_bloc/src/domain/form_operations/form_operation.dart';
import 'package:wyatt_form_bloc/src/domain/input_validators/form_input_validator.dart';

class FormUnion extends FormOperation {
  const FormUnion();

  @override
  WyattForm call(WyattForm a, WyattForm b) {
    final inputs = <
        FormInput<dynamic, FormInputValidator<dynamic, ValidationError>,
            dynamic>>[];

    for (final i in a.inputs) {
      inputs.add(i);
    }

    for (final i in b.inputs) {
      if (!a.containsKey(i.key)) {
        inputs.add(i);
      } else {
        // if a input is pure, but not b, replace a one with b
        if (a.isPureInput(i.key)) {
          inputs
            ..remove(a.inputOf(i.key))
            ..add(i);
        }
      }
    }

    return WyattFormImpl(
      inputs,
      name: a.name,
      validationStrategy: a.formValidationStrategy,
    );
  }
}
