// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_form_bloc/src/core/enums/form_status.dart';
import 'package:wyatt_form_bloc/src/domain/form/wyatt_form.dart';
import 'package:wyatt_form_bloc/src/domain/form_validators/form_validator.dart';

class OnlyRequiredInputValidator extends FormValidator {
  const OnlyRequiredInputValidator() : super();

  @override
  FormStatus validate(WyattForm form) {
    if (isPure(form)) {
      return FormStatus.pure;
    }

    final inputsToTest =
        form.inputs.where((input) => input.metadata.isRequired);

    if (inputsToTest.every((input) => input.validator.pure)) {
      return FormStatus.pure;
    }

    if (inputsToTest.any((input) => input.validator.valid == false)) {
      return FormStatus.invalid;
    }

    return FormStatus.valid;
  }
}
