// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_form_bloc/src/core/enums/validation_error.dart';
import 'package:wyatt_form_bloc/src/domain/input_validators/form_input_validator.dart';

/// {@template confirmed_password}
/// Form input for a confirmed password input.
/// {@endtemplate}
class ConfirmedPassword
    extends EqualityValidator<String, ValidationStandardError> {
  /// {@macro confirmed_password}
  const ConfirmedPassword.pure({this.password = '', String? defaultValue})
      : super.pure(defaultValue);

  /// {@macro confirmed_password}
  const ConfirmedPassword.dirty({required this.password, String? value})
      : super.dirty(value);

  /// The original password.
  final String password;

  @override
  String get another => password;

  @override
  ValidationStandardError get onNotEqual => ValidationStandardError.notEqual;

  @override
  ValidationStandardError get onNull => ValidationStandardError.invalid;

  @override
  List<Object?> get props => super.props + [password];
}
