// ignore_for_file: public_member_api_docs, sort_constructors_first
// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:equatable/equatable.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_form_bloc/src/core/enums/validation_error.dart';
import 'package:wyatt_form_bloc/src/domain/entities/form_input_metadata.dart';
import 'package:wyatt_form_bloc/src/domain/input_validators/form_input_validator.dart';

// ignore: must_be_immutable
class FormInput<
    Value,
    Validator extends FormInputValidator<Value, ValidationError>,
    Extra> extends Equatable implements Entity {
  final String key;
  final Validator validator;
  late FormInputMetadata<Extra> metadata;

  String get name => metadata.name ?? key;

  FormInput(
    this.key,
    this.validator, {
    FormInputMetadata<Extra>? metadata,
  }) {
    this.metadata = metadata ?? FormInputMetadata<Extra>();
  }

  FormInput<Value, Validator, Extra> copyWith({
    String? key,
    Validator? validator,
    FormInputMetadata<Extra>? metadata,
  }) =>
      FormInput(
        key ?? this.key,
        validator ?? this.validator,
        metadata: metadata ?? this.metadata,
      );

  FormInput<Value, Validator, Extra> clone() => copyWith(
        key: key,
        validator: validator,
        metadata: metadata,
      );

  @override
  List<Object?> get props => [key, validator, metadata];

  @override
  String toString() =>
      'FormInput(name: $name, $validator)';
}
