// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'form_input_validator.dart';

abstract class AnyValidator<O, I extends Iterable<O?>,
    E extends ValidationError> extends FormInputValidator<O?, E> {
  const AnyValidator.pure(super.value) : super.pure();
  const AnyValidator.dirty(super.value) : super.dirty();

  I get allChoices;

  bool test(O? element, O value);

  E get onError;

  /// If value is null.
  E get onNull;

  @override
  E? validator(O? value) {
    if (value == null) {
      return onNull;
    }
    if (allChoices.any(
      (element) => test(element, value),
    )) {
      return null;
    }
    return onError;
  }

  @override
  List<Object?> get props => super.props + [allChoices];

  @override
  String toString() => '${super.toString()}, choices: $allChoices';
}
