// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'form_input_validator.dart';

/// {@template nullable_validator}
/// Abstract nullable validator not validating null values.
/// {@endtemplate}
abstract class NullableValidator<O extends Object, E extends ValidationError>
    extends FormInputValidator<O?, E> {
  const NullableValidator.pure(super.value) : super.pure();
  const NullableValidator.dirty(super.value) : super.dirty();

  /// If value is null.
  E get onNull;

  @override
  E? validator(O? value) {
    if (value == null) {
      return onNull;
    }
    return null;
  }
}
