// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_form_bloc/src/core/enums/validation_error.dart';
import 'package:wyatt_form_bloc/src/domain/entities/form_input.dart';
import 'package:wyatt_form_bloc/src/domain/entities/form_input_metadata.dart';
import 'package:wyatt_form_bloc/src/domain/input_validators/form_input_validator.dart';
import 'package:wyatt_form_bloc/src/presentation/presentation.dart';

class InputBuilderMetadata<Cubit extends FormDataCubit, Extra>
    extends StatelessWidget {
  const InputBuilderMetadata({
    required this.field,
    required this.builder,
    super.key,
  });

  final String field;

  final Widget Function(
    BuildContext context,
    Cubit cubit,
    FormDataState state,
    String field,
    FormInput<dynamic, FormInputValidator<dynamic, ValidationError>, dynamic>
        input,
    FormInputMetadata<Extra>? metadata,
  ) builder;

  @override
  Widget build(BuildContext context) => BlocBuilder<Cubit, FormDataState>(
        builder: (context, state) {
          final cubit = context.watch<Cubit>();
          final input = state.form.inputOf(field);
          final metadata = input.metadata as FormInputMetadata<Extra>?;
          return builder.call(
            context,
            cubit,
            state,
            field,
            input,
            metadata,
          );
        },
      );
}
