// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_context.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';

class BodyToJsonMiddleware with OnRequestMiddleware implements Middleware {
  @override
  String getName() => 'BodyToJson';

  @override
  Future<MiddlewareRequest> onRequest(
    MiddlewareContext context,
    MiddlewareRequest request,
  ) async {
    print(
      '${getName()}::OnRequest\n'
      '>> Transforms body in json if Map then update '
      'headers with right content-type',
    );
    final mutation = {
      'content-type': 'application/json; charset=utf-8',
    };
    if (request.body is Map) {
      final Map<String, String> headers = request.headers..addAll(mutation);
      request.modifyRequest(
        request.unfreezedRequest
            .copyWith(headers: headers, body: jsonEncode(request.body)),
      );
    }
    return request;
  }
}
