// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_context.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/models/middleware_response.dart';

class SimpleLoggerMiddleware
    with OnRequestMiddleware, OnResponseMiddleware
    implements Middleware {
  @override
  String getName() => 'SimpleLogger';

  @override
  Future<MiddlewareRequest> onRequest(
    MiddlewareContext context,
    MiddlewareRequest request,
  ) async {
    print(
      '${getName()}::OnRequest\n'
      '>> ${request.method} ${request.url}\n'
      '>> Headers: ${request.headers}\n>> Body: ${request.encodedBody}',
    );
    return request;
  }

  @override
  Future<MiddlewareResponse> onResponse(
    MiddlewareContext context,
    MiddlewareResponse response,
  ) async {
    print(
      '${getName()}::OnResponse\n'
      '>> Status: ${response.status.name.toUpperCase()}\n'
      '>> Length: ${response.contentLength ?? '0'} bytes',
      // '>> Body: ${response.body}',
    );
    return response;
  }
}
