// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_context.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/utils/convert.dart';

class UnsafeAuthMiddleware with OnRequestMiddleware implements Middleware {

  UnsafeAuthMiddleware({
    this.username,
    this.password,
    this.usernameField = 'username',
    this.passwordField = 'password',
  });
  String? username;
  String? password;

  final String usernameField;
  final String passwordField;

  @override
  String getName() => 'UnsafeAuth';

  @override
  Future<MiddlewareRequest> onRequest(
    MiddlewareContext context,
    MiddlewareRequest request,
  ) async {
    if (username == null || password == null) {
      return request;
    }
    final Uri uri =
        request.url + '?$usernameField=$username&$passwordField=$password';
    print(
      '${getName()}::OnRequest\n'
      '>> Append: ?$usernameField=$username&$passwordField=$password',
    );
    request.modifyRequest(request.unfreezedRequest.copyWith(url: uri));
    return request;
  }
}
