// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_http_client/src/middleware.dart';
import 'package:wyatt_http_client/src/models/middleware_context.dart';
import 'package:wyatt_http_client/src/models/middleware_request.dart';
import 'package:wyatt_http_client/src/utils/protocols.dart';

class UriPrefixMiddleware with OnRequestMiddleware implements Middleware {

  UriPrefixMiddleware({
    required this.protocol,
    required this.authority,
  });
  final Protocols protocol;
  final String? authority;

  @override
  String getName() => 'UriPrefix';

  @override
  Future<MiddlewareRequest> onRequest(
    MiddlewareContext context,
    MiddlewareRequest request,
  ) async {
    final Uri uri = Uri.parse('${protocol.scheme}$authority${request.url}');
    print(
      '${getName()}::OnRequest\n'
      '>> From: ${request.url}\n'
      '>> To: $uri',
    );
    request.modifyRequest(request.unfreezedRequest.copyWith(url: uri));
    return request;
  }
}
