// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/domain/entities/component.dart';

abstract class AppBarComponent extends Component {
  final String? title;
  final Widget? leading;
  final List<Widget>? actions;
  const AppBarComponent({
    this.title,
    this.leading,
    this.actions,
    super.key,
  });

  @override
  AppBarComponent? configure({
    String? title,
    Widget? leading,
    List<Widget>? actions,
  });
}

abstract class BottomNavigationBarComponent extends Component {
  final int currentIndex;
  final void Function(BuildContext, int)? onTap;
  const BottomNavigationBarComponent({
    this.onTap,
    this.currentIndex = 0,
    super.key,
  });

  @override
  BottomNavigationBarComponent? configure({
    void Function(BuildContext, int)? onTap,
    int currentIndex = 0,
  });
}

abstract class ErrorWidgetComponent extends Component {
  final String? error;
  const ErrorWidgetComponent({required this.error, super.key});

  @override
  ErrorWidgetComponent? configure({String? error});
}

abstract class LoadingWidgetComponent extends Component {
  final Color? color;
  const LoadingWidgetComponent({required this.color, super.key});

  @override
  LoadingWidgetComponent? configure({Color? color});
}
